/*
 * Copyright 2002 Sun Microsystems Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <libwnck/libwnck.h>
#include <gtk/gtk.h>
#include <gtk/gtkwidget.h>
#include <errno.h>
#include <unistd.h>
#include "workspace-accessible.h"

static void                  wnck_wspace_accessible_class_init      (WnckWSpaceAccessibleClass *klass);
static G_CONST_RETURN gchar* wnck_wspace_accessible_get_name        (AtkObject                 *obj);
static G_CONST_RETURN gchar* wnck_wspace_accessible_get_description (AtkObject *obj);

static void atk_component_interface_init (AtkComponentIface *iface);

static void     wnck_wspace_accessible_get_extents  (AtkComponent *component,
						     gint         *x,
						     gint         *y,
						     gint         *width,
						     gint         *height,
						     AtkCoordType  coords);
static void     wnck_wspace_accessible_get_position (AtkComponent   *component,
	       	                                     gint           *x,
						     gint           *y,
						     AtkCoordType    coords);
static gboolean wnck_wspace_accessible_contains     (AtkComponent *component, 
	                         	             gint          x, 
						     gint          y, 
						     AtkCoordType  coords);
static void     wnck_wspace_accessible_get_size     (AtkComponent *component, 
	     	                                     gint         *width, 
						     gint         *height);

extern GdkRectangle* wnck_wspace_get_rect (WnckPager *pager, 
	      	                           gint       i);

extern AtkObject *aobj_ws[];


GType
wnck_wspace_accessible_get_type (void)
{
  static GType type = 0;

  if (!type)
    {
      static GTypeInfo tinfo = 
      {
        sizeof (WnckWSpaceAccessibleClass),
        (GBaseInitFunc) NULL, /* base init */
        (GBaseFinalizeFunc) NULL, /* base finalize */
        (GClassInitFunc) wnck_wspace_accessible_class_init,
        (GClassFinalizeFunc) NULL, /* class finalize */
        NULL, /* class data */
        sizeof (WnckWSpaceAccessible), /* instance size*/
        0, /* nb preallocs */
        (GInstanceInitFunc) NULL, /* instance init */
        NULL /* value table */
      };

      static const GInterfaceInfo atk_component_info = 
      {
        (GInterfaceInitFunc) atk_component_interface_init,
        (GInterfaceFinalizeFunc) NULL,
        NULL
      };

      type = g_type_register_static (ATK_TYPE_GOBJECT_ACCESSIBLE, "WnckWSpaceAccessible", &tinfo, 0);
      g_type_add_interface_static (type, ATK_TYPE_COMPONENT, &atk_component_info);
  }
  return type;
  
}

static void
atk_component_interface_init (AtkComponentIface *iface)
{
  g_return_if_fail (iface != NULL);

  iface->get_extents = wnck_wspace_accessible_get_extents;
  iface->get_size = wnck_wspace_accessible_get_size;
  iface->get_position = wnck_wspace_accessible_get_position;
  iface->contains = wnck_wspace_accessible_contains;
}

static void 
wnck_wspace_accessible_get_extents (AtkComponent *component,
                                    gint         *x, 
				    gint         *y, 
				    gint         *width,
                                    gint         *height,
				    AtkCoordType  coords)
{
  AtkGObjectAccessible *atk_gobj;
  WnckWorkspace *wspace;
  WnckPager *pager;
  GdkRectangle *rect;
  GtkWidget *widget;
  AtkObject *parent;
  GObject *g_obj;

  gint i;

  g_return_if_fail (WNCK_WSPACE_IS_ACCESSIBLE (component));

  atk_gobj = ATK_GOBJECT_ACCESSIBLE (component);
  g_obj = atk_gobject_accessible_get_object (atk_gobj);
  if (g_obj == NULL)
    return;

  g_return_if_fail (WNCK_IS_WORKSPACE (g_obj));
  wspace = WNCK_WORKSPACE (g_obj);

  parent = atk_object_get_parent (ATK_OBJECT(component));
  widget = GTK_ACCESSIBLE (parent)->widget;
       
  if (widget == NULL)
    {
      /*
       *State is defunct
       */
      return ;
    }

  g_return_if_fail (WNCK_IS_PAGER (widget));
  pager = WNCK_PAGER (widget);

  g_return_if_fail (WNCK_IS_PAGER (pager));

  /*
   * Check which workspace is referred
   */
  for(i = 0; i < GET_MAX_N_WS; i++)
    if (aobj_ws[i] == ATK_OBJECT (component))      
      break;
  
  if(i == GET_MAX_N_WS)
    /*
     * No Accessible object error
     */   
    return;        
  rect = wnck_wspace_get_rect (pager, i); 

  *x = rect->x;
  *y = rect->y;
  *height = rect->height;
  *width = rect->width; 

}
  

static void
wnck_wspace_accessible_get_size (AtkComponent *component,
	            	         gint         *width,
			       	 gint         *height)
{
  AtkCoordType coords; 
  gint x, y;
  wnck_wspace_accessible_get_extents (component, &x, &y, width, height, coords);
}

static void
wnck_wspace_accessible_get_position (AtkComponent *component,
		                     gint         *x,
			      	     gint         *y,
			     	     AtkCoordType  coords)
{
  int width, height;
  wnck_wspace_accessible_get_extents (component, x, y, &width, &height, coords);
}

static gboolean
wnck_wspace_accessible_contains (AtkComponent *component, 
		                 gint          x,
				 gint          y,
				 AtkCoordType  coords)
{
  gint lx, ly, width, height;

  wnck_wspace_accessible_get_extents (component, &lx, &ly, &width, &height, coords);

  /*
   * Check if the specified co-ordinates fall within the workspace.
   */
  if ( (x > lx) && ((lx + width) >= x)  && (y > ly) && ((ly + height) >= ly) )
    return TRUE;
  else
    return FALSE;
  
}

static void
wnck_wspace_accessible_class_init (WnckWSpaceAccessibleClass *klass)
{
  AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
       
  class->get_name = wnck_wspace_accessible_get_name;
  class->get_description = wnck_wspace_accessible_get_description;
}

AtkObject*
wnck_wspace_accessible_new (GObject *obj)
{
  GObject *object;
  AtkObject *atk_object;

  g_return_val_if_fail (WNCK_IS_WORKSPACE (obj), NULL);
  object = g_object_new (WNCK_WSPACE_TYPE_ACCESSIBLE, NULL);
  atk_object = ATK_OBJECT (object);
  atk_object_initialize (atk_object, obj);

  g_return_val_if_fail (ATK_IS_OBJECT (atk_object), NULL);
  return atk_object;
  
}

static G_CONST_RETURN gchar*
wnck_wspace_accessible_get_name (AtkObject *obj)
{
  g_return_val_if_fail (WNCK_WSPACE_IS_ACCESSIBLE (obj), NULL);

  if (obj->name != NULL)
    {
      return obj->name;
    }
  else
    return NULL;
  
}

static G_CONST_RETURN gchar*
wnck_wspace_accessible_get_description (AtkObject *obj)
{
  g_return_val_if_fail (WNCK_WSPACE_IS_ACCESSIBLE (obj), NULL);

  if (obj->description != NULL)
    {
      return obj->description;
    }
  else
    return NULL;
  
}

